<?php
/*
Plugin Name: Markdown on Save Improved
Description: Allows you to compose content in Markdown on a per-item basis from WP admin or mobile/3rd-party apps. The markdown version is stored separately, so you can deactivate this plugin and your posts won't spew out Markdown. Based on <a href="http://wordpress.org/extend/plugins/markdown-osi/">Mark Jaquith's plugin</a>.
Version: 2.0.1
Author: Matt Wiebe
Author URI: http://somadesign.ca/
License: GPL v2
*/

/*
 * Copyright 2011 Matt Wiebe.
 *
 * This software is forked from the original Markdown on Save plugin (c) Mark Jaquith
 * It uses the Markdown Extra and Markdownify libraries. Copyrights and licenses indicated in said libararies.
 *
 */

class SD_Markdown {
	const PM = '_sd_disable_markdown';
	const CONVERT = 'sd_convert_to_markdown';
	var $instance;

	public function __construct() {
		$this->instance =& $this;
		add_action( 'init', array( $this, 'init' ) );
	}

	public function init() {
		load_plugin_textdomain( 'markdown-osi', NULL, basename( dirname( __FILE__ ) ) );
		add_filter( 'wp_insert_post_data', array( $this, 'wp_insert_post_data' ), 10, 2 );
		add_action( 'do_meta_boxes', array( $this, 'do_meta_boxes' ), 20, 2 );
		add_filter( 'edit_post_content', array( $this, 'edit_post_content' ), 10, 2 );
		add_filter( 'edit_post_content_filtered', array( $this, 'edit_post_content_filtered' ), 10, 2 );
		add_action( 'xmlrpc_call', array($this, 'xmlrpc_actions') );
		add_action( 'load-post.php', array( $this, 'load' ) );
		add_action( 'load-post-new.php', array( $this, 'load') );
	}
	
	public function xmlrpc_actions($xmlrpc_method) {
		if ( 'metaWeblog.getRecentPosts' === $xmlrpc_method ) {
			add_action( 'parse_query', array($this, 'make_filterable'), 10, 1 );
		}
		else if ( 'metaWeblog.getPost' === $xmlrpc_method ) {
			$this->prime_post_cache();
		}
	}
	
	private function prime_post_cache() {
		global $wp_xmlrpc_server;
		$params = $wp_xmlrpc_server->message->params;
		$post_id = array_shift($params);
		// prime the post cache
		if ( $this->is_markdown($post_id) ) {
			$post = get_post($post_id);
			if ( ! empty($post->post_content_filtered) )
				$post->post_content = $post->post_content_filtered;
			wp_cache_delete($post->ID, 'posts');
			wp_cache_add($post->ID, $post, 'posts');
		}
	}
	
	public function make_filterable($wp_query) {
		$wp_query->set( 'suppress_filters', false );
		add_action( 'the_posts', array($this, 'the_posts'), 10, 2 );
	}
	
	public function the_posts($posts, $wp_query) {
		foreach ( $posts as $key => $post ) {
			if ( $this->is_markdown($post->ID) ) {
				$posts[$key]->post_content = $posts[$key]->post_content_filtered;
			}
		}
		return $posts;
	}
	
	public function load() {
		if ( ! ( isset( $_GET['post'] ) && ! $this->is_markdown( $_GET['post'] ) ) ) {
			add_filter( 'user_can_richedit', '__return_false', 99 );
		}
	}

	public function wp_insert_post_data( $data, $postarr ) {
		// run once
		remove_filter( 'wp_insert_post_data', array( $this, 'wp_insert_post_data' ), 10, 2 );
		
		// checks
		$nonced = ( isset( $_POST['_sd_markdown_nonce'] ) && wp_verify_nonce( $_POST['_sd_markdown_nonce'], 'sd-markdown-save' ) );
		$disable_ticked = ( $nonced && isset( $_POST['sd_disable_markdown'] ) );
		$disable_comment_inserted = ( false !== stripos($data['post_content'], '<!--no-markdown-->') );
		$do_html_to_markdown = ( $nonced && isset($_POST[self::CONVERT]) );
		$id = ( isset( $postarr['ID']) ) ? $postarr['ID'] : 0;
		
		// Maybe do HTML → Markdown
		if ( $do_html_to_markdown ) 
			$data['post_content'] = $this->html_to_markdown($data['post_content']);
		
		// Make sure markdown processing isn't disabled for this post
		if ( ! ( $disable_ticked || $disable_comment_inserted ) ) {
			$data['post_content_filtered'] = $data['post_content'];
			// Do markdown processing
			$data['post_content'] = $this->process($data['post_content'], $id);
			if ( $id )
				delete_post_meta( $id, self::PM );
		} else {
			$data['post_content_filtered'] = '';
			if ( $id )
				update_post_meta( $id, self::PM, true );
		}
		return $data;
	}
	
	protected function html_to_markdown($content) {
		$content = stripslashes($content);
		$content = wpautop($content);
		require_once( dirname(__FILE__) . '/markdownify/markdownify_extra.php' );
		$md = new Markdownify_Extra(true);
		$content = $md->parseString($content);
		return $content;
	}
	
	protected function process($content, $id) {
		require_once( dirname( __FILE__) . '/markdown-extra/markdown-extra.php' );
		// Links with "quoted titles" bork without stripping slashes.
		// Stripslashes would lose desired backslashes. So, regex.
		$content = preg_replace( '/\\\"/', '"', $content );
		// convert to Markdown, but remove <p> tags for wpautop to work later
		$content = $this->unp( Markdown( $content ) );
		// reference the post_id to make footnote ids unique
		$content = preg_replace('/fn(ref)?:/', "fn$1-$id:", $content);
		return $content;
	}

	public function do_meta_boxes( $type, $context ) {
		// allow disabling for folks who think markdown should always be on.
		if ( defined( 'SD_HIDE_MARKDOWN_BOX') && SD_HIDE_MARKDOWN_BOX )
			return;

		if ( 'side' == $context && in_array( $type, array_keys( get_post_types() ) ) )
			add_meta_box( 'sd-markdown', __( 'Markdown', 'markdown-osi' ), array( $this, 'meta_box' ), $type, 'side', 'high' );
	}

	public function meta_box() {
		global $post;
		wp_nonce_field( 'sd-markdown-save', '_sd_markdown_nonce', false, true );
		echo '<p><input type="checkbox" name="sd_disable_markdown" id="sd_disable_markdown" value="1" ';
		checked( !! get_post_meta( $post->ID, self::PM, true ) );
		echo ' /> <label for="sd_disable_markdown">' . __( 'Disable Markdown formatting', 'markdown-osi' ) . '</label></p>';
		printf( '<p><label><input type="checkbox" name="%s" /> %s</label></p>', self::CONVERT, __('Convert HTML to Markdown (experimental)', 'markdown-osi') );
	}

	private function unp( $content ) {
		return preg_replace( "#<p>(.*?)</p>(\n|$)#", '$1$2', $content );
	}

	private function is_markdown( $id ) {
		return ! get_post_meta( $id, self::PM, true );
	}

	public function edit_post_content( $content, $id ) {
		if ( $this->is_markdown( $id ) ) {
			$post = get_post( $id );
			if ( $post && ! empty( $post->post_content_filtered ) )
				$content = $post->post_content_filtered;
		}
		return $content;
	}

	public function edit_post_content_filtered( $content, $id ) {
		return $content;
	}

}

new SD_Markdown;
